#ifndef CMD_H
#define CMD_H

#include "processing/rotation.h"
#include "result.h"

/**
 * @file cmd.h
 * @brief Command line argument parsing.
 */

/**
 * @brief Command line args required for the program.
 */
typedef struct {
    char const *source_image_path; /**< Path to the source image. */
    char const *output_image_path; /**< Path to the output image. */
    RotationAngle angle;           /**< Rotation angle. */
} CmdArgs;

/**
 * @brief Status of command line args parsing.
 */
typedef enum {
    CMD_ARGS_CORRECT = 0, /**< Command args parsed correctly. */
    CMD_ARGS_NOT_ENOUGH,  /**< Not enough command arguments. */
    CMD_ARGS_BAD_ANGLE,   /**< Incorrect angle value. */
} ArgsParseStatus;

Result(ArgsParseResult, CmdArgs, ArgsParseStatus);

/**
 * @brief Parse command line args.
 *
 * @param argc Number of command line arguments.
 * @param argv Array of command line arguments.
 * @return Args parse result.
 */
ArgsParseResult parse_cmd_args(int argc, char const **argv);

#endif /* CMD_H */
