#ifndef CONTEXT_H
#define CONTEXT_H

#include "_cmd.h"
#include "image.h"
#include "image_io.h"

/**
 * @file context.h
 * @brief Application execution context.
 */

/**
 * @brief Application execution context.
 */
typedef struct {
    ArgsParseResult args;         /**< Parsed command line arguments. */
    ImageOpenResult read_result;  /**< Result of image reading. */
    ImageSaveResult write_result; /**< Result of image saving. */
    MaybeImage rotated_img;       /**< Rotated image. */
    Image source_img;             /**< Source image. */
} Context;

/**
 * @brief Utils for context printing.
 */
#define CONTEXT_FMT                                                            \
    "{\n  args: %d\n  read: %d\n  write: %d\n  rotated: %p\n  source: "        \
    "%p\n}\n"

/**
 * @brief Macro to format context printing arguments.
 */
#define CONTEXT_FMT_ARGS(context)                                              \
    (context).args.status, (context).read_result.status,                       \
        (context).write_result.status, (void *)(context).rotated_img._.pixels, \
        (void *)(context).source_img.pixels
#endif /* CONTEXT_H */
