#ifndef FILE_H
#define FILE_H

#include "result.h"

#include <stdint.h>
#include <stdio.h>

/**
 * @file file.h
 * @brief File handling functions.
 */

/**
 * @brief Data buffer with size.
 */
typedef struct {
    uint8_t *data; /**< Pointer to the data buffer. */
    size_t size;   /**< Size of the data buffer. */
} FileBuffer;

Maybe(FileBuffer);

/**
 * @brief Allocate buffer.
 *
 * @param size Size of the buffer to allocate.
 * @return Maybe file buffer.
 */
MaybeFileBuffer create_buffer(size_t size);

/**
 * @brief Free buffer memory.
 *
 * @param buffer Pointer to the file buffer to be destroyed.
 */
void destroy_buffer(FileBuffer *buffer);

/**
 * @brief Read file result.
 */
IOResult(FileReadResult, FileBuffer);

/**
 * @brief Read whole file content.
 *
 * @param file File pointer.
 * @return File read result.
 */
FileReadResult read_file(FILE *file);

/**
 * @brief Write file result.
 */
IOResult(FileWriteResult, void *);

/**
 * @brief Write buffer data to file.
 *
 * @param file File pointer.
 * @param buffer Data to be written.
 * @return File write result.
 */
FileWriteResult write_file(FILE *file, FileBuffer buffer);

#endif /* FILE_H */
