#ifndef BMP_H
#define BMP_H

#include "_file.h"
#include "image.h"
#include "image_io.h"
#include "result.h"

#include <inttypes.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

/**
 * @file bmp.h
 * @brief BMP image handling functions.
 */

/**
 * @brief Status codes for reading BMP images.
 */
typedef enum {
    FROM_BMP_OK = 0,                  /**< Image read properly. */
    FROM_BMP_INVALID_HEADER,          /**< Unsupported or broken header. */
    FROM_BMP_UNSUPPORTED_FORMAT,      /**< Not BMP format. */
    FROM_BMP_UNSUPPORTED_COMPRESSION, /**< Unsupported compression. */
    FROM_BMP_UNSUPPORTED_COLOR_DEPTH, /**< Unsupported color depth. */
    FROM_BMP_INVALID_PIXELS,          /**< Cannot read pixel data. */
    FROM_BMP_CANNOT_ALLOC_MEMORY,     /**< Not enough memory for image. */
} FromBmpStatus;

/**
 * @brief Read BMP image from file.
 *
 * @param buffer File buffer containing BMP data.
 * @return Image parser result.
 */
ImageParserResult from_bmp(FileBuffer buffer);

/**
 * @brief Status codes for writing BMP images.
 */
typedef enum {
    TO_BMP_OK = 0,              /**< Image read properly. */
    TO_BMP_BAD_IMAGE_DATA,      /**< Image data is null. */
    TO_BMP_WRITE_FAILED,        /**< Writing to file failed. */
    TO_BMP_CANNOT_ALLOC_MEMORY, /**< Not enough memory for image. */
} ToBmpStatus;

/**
 * @brief Write image to file in BMP format.
 *
 * @param img Image to be written.
 * @return Image builder result.
 */
ImageBuilderResult to_bmp(Image img);

#endif /* BMP_H */
