#ifndef IMAGE_H
#define IMAGE_H

#include "result.h"

#include <stdint.h>
#include <stdlib.h>

/**
 * @file image.h
 * @brief Image handling functions.
 */

/**
 * @brief Get pixel at index.
 */
#define pixel_at(img, x, y) ((img).pixels + (x) + (y) * (img).width)

/**
 * @brief Execute expression for each pixel.
 */
#define foreach_pixel(img, expression)                                         \
    for (uint32_t i = 0; i < (img).width; ++i) {                               \
        for (uint32_t j = 0; j < (img).height; ++j) {                          \
            expression;                                                        \
        }                                                                      \
    }

/**
 * @brief Pixel with 1-byte color depth.
 */
typedef struct __attribute__((packed)) {
    uint8_t r, g, b; /**< Red, green, and blue color values. */
} Pixel;

/**
 * @brief RGB image.
 */
typedef struct {
    uint32_t width, height; /**< Image width and height. */
    Pixel *pixels;          /**< Pointer to the pixel data. */
} Image;

Maybe(Image);

/**
 * @brief Return required size for image pixels data.
 */
size_t calc_image_size(uint32_t width, uint32_t height);

/**
 * @brief Create image structure with pre-allocated data.
 * If memory cannot be allocated, image pixels are NULL.
 */
MaybeImage create_image(uint32_t width, uint32_t height);

/**
 * @brief Free image data.
 *
 * @param img Image to be destroyed.
 */
void destroy_image(Image img);

#endif /* IMAGE_H */
