#ifndef IMAGE_IO
#define IMAGE_IO

#include "_file.h"
#include "image.h"
#include "result.h"

#include <stdio.h>

/**
 * @file image_io.h
 * @brief Image input/output functions.
 */

/**
 * @brief Image parser result.
 */
Result(ImageParserResult, Image, uint64_t);

/**
 * @brief Result of image opening.
 */
IOResult(ImageOpenResult, ImageParserResult);

/**
 * @brief Function that converts image data to a specific format.
 */
typedef ImageParserResult ImageParser(FileBuffer buffer);

/**
 * @brief Read and parse image.
 *
 * @param img_path Path to the image file.
 * @param parser Image parser function.
 * @return Image open result.
 */
ImageOpenResult image_open(char const *img_path, ImageParser *parser);

/**
 * @brief Image builder result.
 */
Result(ImageBuilderResult, FileBuffer, uint64_t);

/**
 * @brief Result of image building and saving.
 */
IOResult(ImageSaveResult, ImageBuilderResult);

/**
 * @brief Function that builds data for a specific image file.
 */
typedef ImageBuilderResult ImageBuilder(Image img);

/**
 * @brief Build and save image.
 *
 * @param img_path Path to the output image file.
 * @param img Image to be saved.
 * @param builder Image builder function.
 * @return Image save result.
 */
ImageSaveResult image_save(
    char const *img_path,
    Image img,
    ImageBuilder *builder
);

#endif /* IMAGE_IO_H */
