#ifndef LOG_H
#define LOG_H

#include "string.h"

#include <stdarg.h>
#include <stdio.h>

/**
 * @file log.h
 * @brief Logging functions.
 */

/**
 * @brief Log level.
 */
typedef enum { LOG_ERROR = 0, LOG_INFO, LOG_DEBUG } LogLevel;

/**
 * @brief Set to log everything upon that level.
 */
extern LogLevel LOG_LEVEL;

/**
 * @brief General logger.
 */
void flogf(FILE *stream, LogLevel level, char const *message, ...);

/**
 * @brief Macro for logging an info message.
 */
#define INFO(message) flogf(stdout, LOG_INFO, message)

/**
 * @brief Macro for logging a formatted info message.
 */
#define INFOF(message, ...) flogf(stdout, LOG_INFO, message, __VA_ARGS__)

/**
 * @brief Macro for logging a debug message.
 */
#define DEBUG(message) flogf(stdout, LOG_DEBUG, message)

/**
 * @brief Macro for logging a formatted debug message.
 */
#define DEBUGF(message, ...) flogf(stdout, LOG_DEBUG, message, __VA_ARGS__)

/**
 * @brief Macro for logging an error message.
 */
#define ERROR(message) flogf(stderr, LOG_ERROR, message)

/**
 * @brief Macro for logging a formatted error message.
 */
#define ERRORF(message, ...) flogf(stderr, LOG_ERROR, message, __VA_ARGS__)

#endif /* LOG_H */
