#ifndef ROTATION_H
#define ROTATION_H

#include "image.h"
#include "result.h"

/**
 * @file rotation.h
 * @brief Image rotation functions.
 */

/**
 * @brief Possible image rotation angles.
 */
typedef enum {
    ROT_ANGLE_0 = 0,     /**< No rotation. */
    ROT_ANGLE_90 = 90,   /**< 90-degree rotation. */
    ROT_ANGLE_180 = 180, /**< 180-degree rotation. */
    ROT_ANGLE_270 = 270, /**< 270-degree rotation. */
} RotationAngle;

Maybe(RotationAngle);

/**
 * @brief Parse rotation angle from string.
 *
 * @param str String representation of the rotation angle.
 * @return Maybe rotation angle.
 */
MaybeRotationAngle parse_rotation_angle(char const *str);

/**
 * @brief Rotate image.
 *
 * @param img Image to be rotated.
 * @param angle Rotation angle.
 * @return Maybe rotated image.
 */
MaybeImage rotate_image(Image img, RotationAngle angle);

#endif /* ROTATION_H */
