#include "_file.h"

#include "result.h"

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

MaybeImpl(FileBuffer);

MaybeFileBuffer create_buffer(size_t size) {
    uint8_t *data = malloc(size);
    if (data == NULL) {
        return NoneFileBuffer;
    }
    return SomeFileBuffer((FileBuffer){.data = data, .size = size});
}

void destroy_buffer(FileBuffer *buffer) {
    if (buffer->data != NULL) {
        free(buffer->data);
        buffer->data = NULL;
    }
}

static size_t _get_file_size(FILE *file) {
    fseek(file, 0, SEEK_END);
    const size_t size = ftell(file);
    fseek(file, 0, SEEK_SET);
    return size;
}

FileReadResult read_file(FILE *file) {
    const size_t size = _get_file_size(file);
    MaybeFileBuffer buffer = create_buffer(size);
    if (!buffer.status) {
        return (FileReadResult){IO_READ_ERR};
    }
    const size_t bytes_read = fread(buffer._.data, sizeof(uint8_t), size, file);
    if (bytes_read != size) {
        return (FileReadResult){IO_READ_ERR};
    }
    return (FileReadResult){IO_OK, buffer._};
}

FileWriteResult write_file(FILE *file, FileBuffer buffer) {
    if (buffer.data == NULL) {
        return (FileWriteResult){IO_WRITE_ERR};
    }
    const size_t bytes_written = fwrite(
        buffer.data, sizeof(uint8_t), buffer.size, file
    );
    if (bytes_written != buffer.size) {
        return (FileWriteResult){IO_WRITE_ERR};
    }
    return (FileWriteResult){IO_OK};
}
