#include "image_io.h"

#include "_file.h"

#include <stdio.h>

ImageOpenResult image_open(char const *img_path, ImageParser *parser) {
    FILE *file = fopen(img_path, "rb");
    if (file == NULL) {
        return (ImageOpenResult){IO_OPEN_ERR};
    }
    FileReadResult buffer = read_file(file);
    if (buffer.status != IO_OK) {
        return (ImageOpenResult){buffer.status};
    }

    const ImageParserResult img = parser(buffer._);

    if (fclose(file) != 0) {
        return (ImageOpenResult){IO_CLOSE_ERR};
    }
    destroy_buffer(&buffer._);
    return (ImageOpenResult){IO_OK, img};
}

ImageSaveResult image_save(
    char const *img_path,
    Image img,
    ImageBuilder *builder
) {
    FILE *file = fopen(img_path, "wb");
    if (file == NULL) {
        return (ImageSaveResult){IO_OPEN_ERR};
    }

    ImageBuilderResult buffer = builder(img);
    if (buffer.status != IO_OK) {
        return (ImageSaveResult){buffer.status};
    }

    FileWriteResult written = write_file(file, buffer._);
    if (written.status != IO_OK) {
        return (ImageSaveResult){written.status};
    }
    if (fclose(file) != 0) {
        return (ImageSaveResult){IO_CLOSE_ERR};
    }
    destroy_buffer(&buffer._);
    return (ImageSaveResult){IO_OK, buffer};
}
